@@ -3,12 +3,15 @@ author: Mislav Sever
 
 TitaniumCloud
 A Python module for the ReversingLabs TitaniumCloud REST API-s.
+
+Copyright (c) ReversingLabs International GmbH. 2016-2021
+
+This unpublished material is proprietary to ReversingLabs International GmbH.. All rights reserved.
+Reproduction or distribution, in whole or in part, is forbidden except by express written permission of ReversingLabs International GmbH.
 """
 
 import datetime
 import hashlib
-import json
-import os
 import requests
 
 from ReversingLabs.SDK.helper import ADVANCED_SEARCH_SORTING_CRITERIA, DEFAULT_USER_AGENT, HASH_LENGTH_MAP, \
@@ -23,8 +26,6 @@ JSON = "json"
 
 CLASSIFICATIONS = ("MALICIOUS", "SUSPICIOUS", "KNOWN", "UNKNOWN")
 
-AVAILABLE_PLATFORMS = ("windows7", "windows10")
-
 RHA1_TYPE_MAP = {
     "PE": "pe01",
     "PE+": "pe01",
@@ -48,7 +49,10 @@ class TiCloudAPI(object):
     def __init__(self, host, username, password, verify=True, proxies=None,
                  user_agent=DEFAULT_USER_AGENT, allow_none_return=False):
 
-        self._host = self.__validate_host(host)
+        if host.startswith("http://"):
+            raise WrongInputError("Unsupported protocol definition: TitaniumCloud services can only be used over HTTPS.")
+        self._host = self.__format_url(host)
+
         self._username = username
         self._password = password
         self._credentials = (self._username, self._password)
@@ -67,25 +71,16 @@ class TiCloudAPI(object):
         self._allow_none_return = allow_none_return
 
     @staticmethod
-    def __validate_host(host):
+    def __format_url(host):
         """Returns a formatted host URL including the protocol prefix.
             :param host: URL string
             :type host: str
             :returns: formatted URL string
             :rtype: str
         """
-        if not isinstance(host, str):
-            raise WrongInputError("host parameter must be string.")
-
-        if host.startswith("http://"):
-            raise WrongInputError("Unsupported protocol definition: "
-                                  "TitaniumCloud services can only be used over HTTPS.")
-
         if not host.startswith("https://"):
             host = "https://{host}".format(host=host)
 
-        host = host.rstrip("/")
-
         return host
 
     def _get_request(self, url):
@@ -105,13 +100,12 @@ class TiCloudAPI(object):
 
         return response
 
-    def _post_request(self, url, post_json=None, data=None):
+    def _post_request(self, url, post_json=None):
         """A generic POST request method for all ticloud module classes.
             :param url: request URL
             :type url: str
             :param post_json: JSON body
             :type post_json: dict
-            :param data: data to send
             :return: response
             :rtype: requests.Response
         """
@@ -119,7 +113,6 @@ class TiCloudAPI(object):
             url=url,
             auth=self._credentials,
             json=post_json,
-            data=data,
             verify=self._verify,
             proxies=self._proxies,
             headers=self._headers
@@ -345,14 +338,14 @@ class FileAnalysis(TiCloudAPI):
         return response
 
     @staticmethod
-    def extract_uri_list_from_report(report_dict):
-        """Return a list of all the URIs from a file analysis report dictionary.
-            :param report_dict: file analysis report dictionary
-            :type report_dict: dict
+    def extract_uri_list_from_report(reports_dict):
+        """Return a list of all the URIs from a list of results (entries).
+            :param reports_dict: list of results (entries)
+            :type reports_dict: dict
             :return: list of uris
             :rtype: list
         """
-        if not isinstance(report_dict, dict):
+        if not isinstance(reports_dict, dict):
             raise WrongInputError("reports_dict parameter must be a dictionary.")
 
         sha1_key = SHA1
@@ -414,67 +407,59 @@ class FileAnalysis(TiCloudAPI):
 
         uris = []
 
-        sample = report_dict.get("rl", {}).get("sample", {})
-        for entry in sample.get("analysis", {}).get("entries", []):
-            for interesting_string in entry.get("tc_report", {}).get("interesting_strings", []):
-                for value in interesting_string['values']:
-                    uris.append({
-                        sha1_key: sample[SHA1],
-                        from_key: "interesting_strings",
-                        category_key: interesting_string["category"],
-                        uri_key: value
-                    })
+        for report in reports_dict:
+            for entry in report.get("analysis", {}).get("entries", []):
+                for interesting_string in entry.get("tc_report", {}).get("interesting_strings", []):
+                    for value in interesting_string['values']:
+                        uris.append({
+                            sha1_key: report[SHA1],
+                            from_key: "interesting_strings",
+                            category_key: interesting_string["category"],
+                            uri_key: value
+                        })
 
-            for property in entry.get("tc_report", {}).get("info", {}).get("package", {}).get("properties", []):
-                if property.get("name", "").startswith("botServer"):
-                    uris.append({
-                        sha1_key: sample[SHA1],
-                        from_key: "package",
-                        category_key: "uri",
-                        uri_key: property["value"]
-                    })
+                for property in entry.get("tc_report", {}).get("info", {}).get("package", {}).get("properties", []):
+                    if property.get("name", "").startswith("botServer"):
+                        uris.append({
+                            sha1_key: report[SHA1],
+                            from_key: "package",
+                            category_key: "uri",
+                            uri_key: property["value"]
+                        })
 
-        for entry in sample.get("sources", {}).get("entries", []):
-            if "domain" in entry:
-                uris.append({
-                    sha1_key: sample[SHA1],
-                    from_key: "sources",
-                    category_key: "domain",
-                    uri_key: entry["domain"].get("name")
-                })
-            for property in entry.get("properties", []):
-                if property["name"] == "url" and property["value"] != '':
+            for entry in report.get("sources", {}).get("entries", []):
+                if "domain" in entry:
                     uris.append({
-                        sha1_key: sample[SHA1],
+                        sha1_key: report[SHA1],
                         from_key: "sources",
-                        category_key: "url",
-                        uri_key: property["value"]
+                        category_key: "domain",
+                        uri_key: entry["domain"].get("name")
                     })
+                for property in entry.get("properties", []):
+                    if property["name"] == "url" and property["value"] != '':
+                        uris.append({
+                            sha1_key: report[SHA1],
+                            from_key: "sources",
+                            category_key: "url",
+                            uri_key: property["value"]
+                        })
 
-        for entry in sample.get("dynamic_analysis", {}).get("entries", []):
-            if "dynamic_analysis_report_joe_sandbox" in entry:
-                report_dict = entry["dynamic_analysis_report_joe_sandbox"]
-                if "network" in report_dict:
-                    uris.extend(network_uris(report_dict["network"],
-                                             "dynamic_analysis_report_joe_sandbox",
-                                             sample[SHA1]))
+            for entry in report.get("dynamic_analysis", {}).get("entries", []):
+                if "dynamic_analysis_report_joe_sandbox" in entry:
+                    reports_dict = entry["dynamic_analysis_report_joe_sandbox"]
+                    if "network" in reports_dict:
+                        uris.extend(network_uris(reports_dict["network"],
+                                                 "dynamic_analysis_report_joe_sandbox",
+                                                 report[SHA1]))
 
-            if "dynamic_analysis_report" in entry:
-                report_dict = entry["dynamic_analysis_report"]
-                if "network" in report_dict:
-                    uris.extend(network_uris(report_dict["network"],
-                                             "dynamic_analysis_report",
-                                             sample[SHA1]))
+                if "dynamic_analysis_report" in entry:
+                    reports_dict = entry["dynamic_analysis_report"]
+                    if "network" in reports_dict:
+                        uris.extend(network_uris(reports_dict["network"],
+                                                 "dynamic_analysis_report",
+                                                 report[SHA1]))
 
-        transitional_dict = {}
-        for item in uris:
-            item_str = json.dumps(item)
-            transitional_dict[item_str] = item_str
-
-        deduplicated_uris = []
-        for key, _ in transitional_dict.items():
-            item = json.loads(key)
-            deduplicated_uris.append(item)
+        deduplicated_uris = list(set(uris))
 
         return deduplicated_uris
 
@@ -709,7 +694,7 @@ class RHA1Analytics(TiCloudAPI):
             )
 
             post_json = {"rl": {"query": {"rha1_type": rha1_type, "response_format": "json",
-                                          "extended": extended_results, "hashes": hash_input}}}
+                                "extended": extended_results, "hashes": hash_input}}}
 
             response = self._post_request(url=url, post_json=post_json)
 
@@ -1549,371 +1534,6 @@ class AnalyzeURL(TiCloudAPI):
         return response
 
 
-class FileUpload(TiCloudAPI):
-    """TCA-0202 and TCA-0203"""
-
-    __UPLOAD_ENDPOINT = "/api/spex/upload"
-
-    def __init__(self, host, username, password, verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT,
-                 allow_none_return=False):
-        super(FileUpload, self).__init__(host, username, password, verify, proxies, user_agent=user_agent,
-                                         allow_none_return=allow_none_return)
-
-        self._url = "{host}{{endpoint}}".format(host=self._host)
-
-    def upload_sample_from_path(self, file_path, sample_name=None, sample_domain=None):
-        """Accepts a file path string and uploads the desired file to the File Upload API.
-            :param file_path: file path string
-            :type file_path: str
-            :param sample_name: optional name of the sample to be displayed in the cloud
-            :type sample_name: str
-            :param sample_domain: optional domain string of the sample to be displayed in the cloud
-            :type sample_domain: str
-            :return: response
-            :rtype: requests.Response
-        """
-        if not isinstance(file_path, str):
-            raise WrongInputError("file_path parameter must be integer.")
-
-        if sample_name:
-            if not isinstance(sample_name, str):
-                raise WrongInputError("sample_name parameter must be string.")
-        else:
-            sample_name = self.__get_sample_name(file_path=file_path)
-
-        try:
-            file_handle = open(file_path, "rb")
-        except IOError as error:
-            raise WrongInputError("Error while opening file in 'rb' mode - {error}".format(error=str(error)))
-
-        response = self.upload_sample_from_file(
-            file_handle=file_handle,
-            sample_name=sample_name,
-            sample_domain=sample_domain
-        )
-
-        return response
-
-    def upload_sample_from_file(self, file_handle, sample_name=None, sample_domain=None):
-        """Accepts an open file handle and uploads the desired file to the File Upload API.
-            :param file_handle: open file
-            :type file_handle: file or BinaryIO
-            :param sample_name: optional name of the sample to be displayed in the cloud
-            :type sample_name: str
-            :param sample_domain: optional domain string of the sample to be displayed in the cloud
-            :type sample_domain: str
-            :return: response
-            :rtype: requests.Response
-        """
-        if not hasattr(file_handle, "read"):
-            raise WrongInputError("file_handle parameter must be a file open in 'rb' mode.")
-
-        if sample_name:
-            if not isinstance(sample_name, str):
-                raise WrongInputError("sample_name parameter must be string.")
-        else:
-            sample_name = "sample"
-
-        if sample_domain:
-            if not isinstance(sample_domain, str):
-                raise WrongInputError("sample_domain parameter must be string.")
-        else:
-            sample_domain = ""
-
-        file_sha1 = calculate_hash(
-            data_input=file_handle,
-            hashing_algorithm="sha1"
-        )
-
-        file_handle.seek(0)
-
-        endpoint = "{endpoint_base}/{sha1}".format(
-            endpoint_base=self.__UPLOAD_ENDPOINT,
-            sha1=file_sha1
-        )
-
-        url = self._url.format(endpoint=endpoint)
-
-        self._headers["Content-Type"] = "application/octet-stream"
-
-        response = self._post_request(url=url, data=file_handle)
-
-        self._raise_on_error(response)
-
-        response = self.__upload_meta(
-            url=url,
-            sample_name=sample_name,
-            sample_domain=sample_domain
-        )
-
-        return response
-
-    def __upload_meta(self, url, sample_name, sample_domain):
-        """Private method for setting up and uploading metadata of a sample uploaded to the File Upload API.
-            :param url: URL used for sample upload
-            :type url: str
-            :param sample_name: optional name of the sample to be displayed in the cloud
-            :type sample_name: str
-            :param sample_domain: optional domain string of the sample to be displayed in the cloud
-            :type sample_domain: str
-            :return: response
-            :rtype: requests.Response
-        """
-        meta_url = "{url}/meta".format(url=url)
-
-        meta_xml = "<rl><properties><property><name>file_name</name><value>{sample_name}</value></property>" \
-                   "</properties><domain>{domain}</domain></rl>".format(domain=sample_domain, sample_name=sample_name)
-
-        response = self._post_request(
-            url=meta_url,
-            data=meta_xml
-        )
-
-        self._raise_on_error(response)
-
-        return response
-
-    @staticmethod
-    def __get_sample_name(file_path):
-        """Private method for parsing the name of the sample if one is not provided.
-            :param file_path: file path string
-            :type file_path: str
-            :return: parsed sample name
-            :rtype: str
-        """
-        if "nt" in os.name:
-            split_path = file_path.split("\\")
-        else:
-            split_path = file_path.split("/")
-
-        sample_name = split_path[-1]
-
-        if len(sample_name) == 0:
-            sample_name = "sample"
-
-        return sample_name
-
-
-class DynamicAnalysis(TiCloudAPI):
-    """TCA-0207 and TCA-0106"""
-
-    __DETONATE_SAMPLE_ENDPOINT = "/api/dynamic/analysis/analyze/v1/query/json"
-    __GET_RESULTS_ENDPOINT = "/api/dynamic/analysis/report/v1/query/sha1"
-
-    def __init__(self, host, username, password, verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT,
-                 allow_none_return=False):
-        super(DynamicAnalysis, self).__init__(host, username, password, verify, proxies, user_agent=user_agent,
-                                              allow_none_return=allow_none_return)
-
-        self._url = "{host}{{endpoint}}".format(host=self._host)
-
-    def detonate_sample(self, sample_sha1, platform):
-        """Submits a sample available in the cloud for dynamic analysis and returns processing info.
-            :param sample_sha1: SHA-1 hash of the sample
-            :type sample_sha1: str
-            :param platform: desired platform on which the sample will be detonated; see available platforms
-            :type platform: str
-            :return: response
-            :rtype: requests.Response
-        """
-        validate_hashes(
-            hash_input=[sample_sha1],
-            allowed_hash_types=(SHA1,)
-        )
-
-        if platform not in AVAILABLE_PLATFORMS:
-            raise WrongInputError("platform parameter must be one "
-                                  "of the following values: {platforms}".format(platforms=AVAILABLE_PLATFORMS))
-
-        url = self._url.format(endpoint=self.__DETONATE_SAMPLE_ENDPOINT)
-
-        post_json = {"rl": {"sha1": sample_sha1, "platform": platform, "response_format": "json"}}
-
-        response = self._post_request(
-            url=url,
-            post_json=post_json
-        )
-
-        self._raise_on_error(response)
-
-        return response
-
-    def get_dynamic_analysis_results(self, sample_hash, latest=False, analysis_id=None):
-        """Returns dynamic analysis results for a desired sample.
-        The analysis of the selected sample must be finished for the results to be available.
-            :param sample_hash: SHA-1 hash of a desired sample
-            :type sample_hash: str
-            :param latest: return only the latest analysis results
-            :type latest: bool
-            :param analysis_id: return only the results of this analysis
-            :type analysis_id: str
-            :return: response
-            :rtype: requests.Response
-        """
-        validate_hashes(
-            hash_input=[sample_hash],
-            allowed_hash_types=(SHA1,)
-        )
-
-        endpoint = "{endpoint_base}/{sample_hash}".format(
-            endpoint_base=self.__GET_RESULTS_ENDPOINT,
-            sample_hash=sample_hash
-        )
-
-        if latest:
-            if analysis_id:
-                raise WrongInputError("Can not use analysis_id because latest is being used.")
-
-            if str(latest).lower() != "true":
-                raise WrongInputError("latest parameter must be boolean.")
-
-            endpoint = "{endpoint}/latest".format(endpoint=endpoint)
-
-        if analysis_id:
-            if not isinstance(analysis_id, str):
-                raise WrongInputError("analysis_id parameter bust be string.")
-
-            endpoint = "{endpoint}/{analysis_id}".format(
-                endpoint=endpoint,
-                analysis_id=analysis_id
-            )
-
-        endpoint = "{endpoint}?format=json".format(endpoint=endpoint)
-
-        url = self._url.format(endpoint=endpoint)
-
-        response = self._get_request(url=url)
-
-        self._raise_on_error(response)
-
-        return response
-
-
-class CertificateAnalytics(TiCloudAPI):
-    """TCA-0502"""
-
-    __SINGLE_QUERY_ENDPOINT = "/api/certificate/analytics/v1/query/thumbprint"
-    __BULK_QUERY_ENDPOINT = "/api/certificate/analytics/v1/query/thumbprint/json"
-
-    def __init__(self, host, username, password, verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT,
-                 allow_none_return=False):
-        super(CertificateAnalytics, self).__init__(host, username, password, verify, proxies, user_agent=user_agent,
-                                                   allow_none_return=allow_none_return)
-
-        self._url = "{host}{{endpoint}}".format(host=self._host)
-
-    def get_certificate_analytics(self, certificate_thumbprints):
-        """Accepts a certificate hash thumbprint and returns certificate analytics results.
-            :param certificate_thumbprints: hash string or list of hash strings
-            :type certificate_thumbprints: str or list[str]
-            :return: response
-            :rtype: requests.Response
-        """
-        if isinstance(certificate_thumbprints, str):
-            validate_hashes(
-                hash_input=[certificate_thumbprints],
-                allowed_hash_types=(MD5, SHA1, SHA256)
-            )
-
-            endpoint = "{endpoint_base}/{thumbprint}?format=json".format(
-                endpoint_base=self.__SINGLE_QUERY_ENDPOINT,
-                thumbprint=certificate_thumbprints
-            )
-
-            url = self._url.format(endpoint=endpoint)
-
-            response = self._get_request(url=url)
-
-        elif isinstance(certificate_thumbprints, list) and len(certificate_thumbprints) > 0:
-            validate_hashes(
-                hash_input=certificate_thumbprints,
-                allowed_hash_types=(MD5, SHA1, SHA256)
-            )
-
-            url = self._url.format(endpoint=self.__BULK_QUERY_ENDPOINT)
-
-            post_json = {"rl": {"query": {"thumbprints": certificate_thumbprints, "format": "json"}}}
-
-            response = self._post_request(url=url, post_json=post_json)
-
-        else:
-            raise WrongInputError("Only hash string or list of hash strings are allowed as the thumbprint parameter.")
-
-        self._raise_on_error(response)
-
-        return response
-
-
-class RansomwareIndicators(TiCloudAPI):
-    """Ransomware Indicators Feed"""
-
-    __FEED_ENDPOINT = "/api/public/v1/ransomware/indicators?withHealth={health}&tagFormat={tag_format}&" \
-                      "hours={hours_back}&indicatorTypes={indicator_types}&onlyFreemium={only_freemium}"
-
-    def __init__(self, host, username, password, verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT,
-                 allow_none_return=False):
-        super(RansomwareIndicators, self).__init__(host, username, password, verify, proxies, user_agent=user_agent,
-                                                   allow_none_return=allow_none_return)
-
-        self._url = "{host}{{endpoint}}".format(host=self._host)
-        self._allowed_indicator_types = ("ipv4", "hash", "domain", "uri")
-
-    def get_indicators(self, hours_back, indicator_types, tag_format="dict", health_check=0, only_freemium=0):
-        """Accepts a list of indicator type strings and an integer for historical hours.
-        Returns indicators of ransomware and related tools.
-            :param hours_back: historical hours - from this moment back
-            :type hours_back: int
-            :param indicator_types: a list of indicator types to fetch; possible values are 'ipv4', 'hash', 'domain', 'uri'
-            :type indicator_types: list[str]
-            :param tag_format: response format; default is 'dict'
-            :type tag_format: str
-            :param health_check: defines whether this request is an API health check;
-            possible values are 0 and 1; default is 0
-            :type health_check: int
-            :param only_freemium: return only freemium indicators of all types;
-            possible values are 0 and 1: default is 0; if set to 1, parameter indicator_types is ignored
-            :type only_freemium: int
-            :return: response
-            :rtype: requests.Response
-        """
-        if not (isinstance(hours_back, int) and 1 <= hours_back <= 48):
-            raise WrongInputError("hours_back parameter must be integer with a value between 1 and 48.")
-
-        try:
-            lowered_indicator_types = [indicator_type.lower() for indicator_type in indicator_types]
-
-        except (AttributeError, TypeError):
-            raise WrongInputError("indicator_types parameter must be a list of strings.")
-
-        if not all(element in self._allowed_indicator_types for element in lowered_indicator_types):
-            raise WrongInputError("Only the following values are allowed as indicator types: "
-                                  "{indicator_types}".format(indicator_types=self._allowed_indicator_types))
-
-        indicator_types = ",".join(lowered_indicator_types)
-
-        if health_check not in (0, 1):
-            raise WrongInputError("health_check parameter must be 0 or 1.")
-
-        if only_freemium not in (0, 1):
-            raise WrongInputError("only_freemium parameter must be 0 or 1.")
-
-        endpoint = self.__FEED_ENDPOINT.format(
-            health=health_check,
-            tag_format=tag_format,
-            hours_back=hours_back,
-            indicator_types=indicator_types,
-            only_freemium=only_freemium
-        )
-
-        url = self._url.format(endpoint=endpoint)
-
-        response = self._get_request(url=url)
-
-        self._raise_on_error(response)
-
-        return response
-
-
 def _update_hash_object(input_source, hash_object):
     """Accepts a string or an opened file in 'rb' mode and a created hashlib hash object and
     returns an updated hashlib hash object.
